TYPE
  LOGLEVEL : (CRITICAL, WARNING, INFO, DEBUG) := INFO;
END_TYPE

FUNCTION_BLOCK LOGGER
  VAR_INPUT
    TRIG : BOOL;
    MSG : STRING;
    LEVEL : LOGLEVEL := INFO;
  END_VAR
  VAR
    TRIG0 : BOOL;
  END_VAR

  IF TRIG AND NOT TRIG0 THEN
  {{
   LogMessage(GetFbVar(LEVEL),(char*)GetFbVar(MSG, .body),GetFbVar(MSG, .len));
  }}
  END_IF;
  TRIG0:=TRIG;
END_FUNCTION_BLOCK



FUNCTION_BLOCK python_eval
  VAR_INPUT
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    STATE : DWORD;
    BUFFER : STRING;
    PREBUFFER : STRING;
    TRIGM1 : BOOL;
    TRIGGED : BOOL;
  END_VAR

  {extern void __PythonEvalFB(int, PYTHON_EVAL*);__PythonEvalFB(0, data__);}
END_FUNCTION_BLOCK

FUNCTION_BLOCK python_poll
  VAR_INPUT
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    STATE : DWORD;
    BUFFER : STRING;
    PREBUFFER : STRING;
    TRIGM1 : BOOL;
    TRIGGED : BOOL;
  END_VAR

  {extern void __PythonEvalFB(int, PYTHON_EVAL*);__PythonEvalFB(1,(PYTHON_EVAL*)(void*)data__);}
END_FUNCTION_BLOCK

FUNCTION_BLOCK python_gear
  VAR_INPUT
    N : UINT;
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    py_eval : python_eval;
    COUNTER : UINT;
    ADD10_OUT : UINT;
    EQ13_OUT : BOOL;
    SEL15_OUT : UINT;
    AND7_OUT : BOOL;
  END_VAR

  ADD10_OUT := ADD(COUNTER, 1);
  EQ13_OUT := EQ(N, ADD10_OUT);
  SEL15_OUT := SEL(EQ13_OUT, ADD10_OUT, 0);
  COUNTER := SEL15_OUT;
  AND7_OUT := AND(EQ13_OUT, TRIG);
  py_eval(TRIG := AND7_OUT, CODE := CODE);
  ACK := py_eval.ACK;
  RESULT := py_eval.RESULT;
END_FUNCTION_BLOCK


FUNCTION_BLOCK BELL
  VAR
    TON0 : TON;
    TON1 : TON;
    TON2 : TON;
    TON3 : TON;
  END_VAR
  VAR_INPUT
    SWITCH : BOOL;
    TM0_V0 : TIME;
    TM0_V1 : TIME;
    TM0_V2 : TIME;
    TM0_V3 : TIME;
  END_VAR
  VAR
    TIME_UP0 : BOOL;
    TIME_UP1 : BOOL;
    TIME_UP2 : BOOL;
    TIME_UP3 : BOOL;
  END_VAR
  VAR_OUTPUT
    OUT : BOOL;
  END_VAR

  TON0(IN := NOT(TIME_UP1) AND SWITCH, PT := TM0_V0);
  TIME_UP0 := TON0.Q;
  TON1(IN := TIME_UP0 AND SWITCH, PT := TM0_V1);
  TIME_UP1 := TON1.Q;
  TON2(IN := NOT(TIME_UP3) AND SWITCH, PT := TM0_V2);
  TIME_UP2 := TON2.Q;
  TON3(IN := TIME_UP2 AND SWITCH, PT := TM0_V3);
  TIME_UP3 := TON3.Q;
  OUT := TIME_UP2 AND TIME_UP0 AND SWITCH;
END_FUNCTION_BLOCK

PROGRAM program0
  VAR
    BELL0 : BELL;
    SWITCH1 : BOOL;
    COIL1 : BOOL;
    VALUE1 : TIME := TIME#1s;
    VALUE2 : TIME := TIME#1s;
    VALUE3 : TIME := T#0.1s;
    VALUE4 : TIME := T#0.1s;
  END_VAR

  BELL0(SWITCH := SWITCH1, TM0_V0 := VALUE1, TM0_V1 := VALUE2, TM0_V2 := VALUE3, TM0_V3 := VALUE4);
  COIL1 := BELL0.OUT;
END_PROGRAM


CONFIGURATION Config0

  RESOURCE Res0 ON PLC
    TASK task0(INTERVAL := T#20ms,PRIORITY := 0);
    PROGRAM instance0 WITH task0 : program0;
  END_RESOURCE
END_CONFIGURATION
